'use strict';

/**
 * Copyright 2015 Urban Airship Inc. and Contributors. Subject to the LICENSE
 * file at the top-level directory of this distribution and at
 * https://github.com/urbanairship/frock/blob/master/LICENSE
 */
var http = require('http');

var bole = require('bole');
var arrayify = require('arrify');
var enableDestroy = require('server-destroy');

var _require = require('./middleware'),
    utilMiddleware = _require.utilMiddleware,
    logMiddleware = _require.logMiddleware;

var _require2 = require('./utils'),
    processMiddleware = _require2.processMiddleware,
    handleServerError = _require2.handleServerError,
    createConnectionFilter = _require2.createConnectionFilter;

var log = bole('frock/core-httpsever');

module.exports = createHttpServer;

function createHttpServer(frock, config, globalConfig, ready) {
  var deter = createConnectionFilter(frock, config, globalConfig, onWhitelistFail);

  if (!config.middleware) {
    config.middleware = [];
  }

  config.middleware.unshift({ handler: utilMiddleware });

  var router = frock.router(defaultRoute, config.baseUrl);
  var perServerMiddleware = processMiddleware(frock, bole('middleware:' + config.port + '>'), config.options, config.middleware, deter(router));
  var server = http.createServer(perServerMiddleware);

  var serverRoutes = config.routes || [];
  var boundHandlers = [];
  var errors = [];
  var boundMiddlewares = [perServerMiddleware];

  // handle validation cases
  if (!serverRoutes.length) {
    log.warn('server ' + config.port + ' has no routes');
  }

  if (!config.port || !Number.isInteger(config.port)) {
    var error = new Error('no port defined for server, stopping setup early');

    log.error(error, config);

    return ready(error);
  }

  serverRoutes.forEach(createRoute);

  // start our servers
  server.on('error', handleServerError(log, config));
  server.listen(config.port);
  enableDestroy(server);

  ready(errors.length ? errors : null, {
    server: server,
    handlers: boundHandlers,
    middlewares: boundMiddlewares,
    port: config.port
  });

  log.info('started server ' + config.port);

  function createRoute(route) {
    var methods = arrayify(route.methods).map(function (m) {
      return m.toLowerCase();
    });

    try {
      frock.handlers.register(route.handler);
    } catch (e) {
      errors.push(e);
      log.error('error registering handler ' + route.handler, e);
    }

    methods.forEach(function (method) {
      var logId = route.handler + ':' + config.port + '>';
      var handlerFn = frock.handlers.get(route.handler);

      var handler = void 0;

      if (!handlerFn) {
        var err = new Error(method + ' handler requested ' + route.handler + ', which wasn\'t available.');

        log.error(err);
        errors.push(err);

        return;
      }

      if (!route.options) {
        route.options = {};
      }

      if (!route.middleware) {
        route.middleware = [];
      }

      route.middleware.unshift({ handler: logMiddleware });

      // save the path for plugins that need to know it
      route.options._path = route.path;
      route.options._middleware = route.middleware;

      handler = handlerFn(frock, bole(logId), route.options, route.db ? frock.dbs.register(route.db) : null);

      var middlewareProcessor = processMiddleware(frock, bole(logId), route.options, route.middleware, handler);

      router[method](route.path, middlewareProcessor);

      boundHandlers.push(handler);
      boundMiddlewares.push(middlewareProcessor);

      log.debug('added route [' + method + ':' + route.handler + '] ' + route.path);
    });
  }
}

// testing exports
createHttpServer._defaultRoute = defaultRoute;
createHttpServer._onWhitelistFail = onWhitelistFail;

function defaultRoute(req, res) {
  var msg = 'no route configured for ' + req.url;

  res.statusCode = 404;
  res.end(msg);
  log.info(msg);
}

function onWhitelistFail(req, res) {
  var msg = 'access from non-whitelisted, or from blacklisted address. ' + 'see https://github.com/urbanairship/frock/blob/master/docs/frockfile.md#connection-object-optional ' + 'for further details.';

  res.statusCode = 403;
  res.end(msg);
  log.info(msg);
}